;; Generated package description from test-simple.el  -*- no-byte-compile: t -*-
(define-package "test-simple" "1.3.1.0.20251124.94637" "Simple Unit Test Framework for Emacs Lisp" '((cl-lib "0")) :commit "d5ac766eba9e28e4b1848c45cbf6049046ca00ed" :authors '(("Rocky Bernstein" . "rocky@gnu.org")) :maintainer '("Rocky Bernstein" . "rocky@gnu.org") :keywords '("unit-test") :url "https://github.com/rocky/emacs-test-simple")
